//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function () {
    if (window.WebViewJavascriptBridge) {
        return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'elemejsbridge';
    var HANDLER = '_handler';
    var RESPONSE = '_response';
    var INTERFACE = '_interface';

    var responseCallbacks = {};
    var uniqueId = 1;

    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    //set default messageHandler
    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    function send(data, responseCallback) {
        _doSend({
            data: data
        }, responseCallback, HANDLER);
    }

    function inject(obj, methods) {
        window[obj] = {};
        var jsObj = window[obj];
        var l = methods.length;
        for (var i = 0; i < l; i++) {
            (function () {
                var method = methods[i];
                var jsMethod = method.replace(new RegExp(":", "g"), "");
                jsObj[jsMethod] = function () {
                    if (arguments.length > 2) {
                        throw new Error('arguments Error');
                    }
                    if (arguments.length == 1) {
                        if (typeof arguments[0] == "function") {
                            return _callJavascriptInterface(obj, method, null, arguments[0]);
                        }
                        return _callJavascriptInterface(obj, method, arguments[0], null);
                    }

                    if (arguments.length == 0) {
                        return _callJavascriptInterface(obj, method, null, null);
                    }

                    return _callJavascriptInterface(obj, method, arguments[0], arguments[1]);
                };
            })();
        }
    }

    function injectEvent() {
        var doc = document;
        var readyEvent = doc.createEvent('Events');
        readyEvent.initEvent('WebViewJavascriptBridgeInjectFinishedReady');
        doc.dispatchEvent(readyEvent);
    }

    function _callJavascriptInterface(obj, method, data, responseCallback) {
        if (data) {
            _doSend({
                obj: obj,
                method: method,
                data: data
            }, responseCallback, INTERFACE);
        } else {
            _doSend({
                obj: obj,
                method: method
            }, responseCallback, INTERFACE);
        }

    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback, HANDLER);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback, type) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

        sendMessageQueue.push(message);
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        var url = CUSTOM_PROTOCOL_SCHEME + '://return/' + type + '/' + encodeURIComponent(messageQueueString);
        messagingIframe.src = url;
    }

    //提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function () {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(message.data);
                delete responseCallbacks[message.responseId];
            } else {
                //直接发送
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function (data) {
                        _doSend({
                            responseId: callbackResponseId,
                            data: data
                        }, null, RESPONSE);
                    };
                }

                var handler = WebViewJavascriptBridge._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                //查找指定handler
                try {
                    handler(message.data, responseCallback);
                } catch (exception) {
                    if (typeof console != 'undefined') {
                        console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }

    //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
    function _handleMessageFromNative(messageJSON) {
        if (receiveMessageQueue) {
            receiveMessageQueue.push(messageJSON);
        } else {
            _dispatchMessageFromNative(messageJSON);
        }
    }

    var WebViewJavascriptBridge = window.WebViewJavascriptBridge = {
        init: init,
        send: send,
        inject: inject,
        injectEvent: injectEvent,
        registerHandler: registerHandler,
        callHandler: callHandler,
        _handleMessageFromNative: _handleMessageFromNative
    };

    var doc = document;
    _createQueueReadyIframe(doc);
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    doc.dispatchEvent(readyEvent);
})();